@@ -352,7 +352,7 @@ class DiscreteVariableEditor(VariableEditor):
         """Clear the model state.
         """
         VariableEditor.clear(self)
-        self.values_model.wrap([])
+        self.values_model.clear()
 
     @Slot()
     def on_values_changed(self):
@@ -473,7 +473,7 @@ class OWEditDomain(widget.OWWidget):
 
     def _initialize(self):
         domain = self.data.domain
-        self.input_vars = tuple(domain) + domain.metas
+        self.input_vars = domain.variables + domain.metas
         self.domain_model[:] = list(self.input_vars)
 
     def _restore(self):
